#!/bin/sh

# This file configuring and start Wireless AP mode

# Command line arguments:
#   $1 interface name
#   $2 wireless ssid
#   $3 vendor element

#############################
#########  Params  ##########
#############################

echo Aqara_Hub-`cat /sys/class/net/wlan0/address | sed 's/://g'  | cut -c 9-` > wlan_hostname
INTERFACE="$1"
#SSID="$2"
model=`cat /lumi/conf/device.conf | grep -v ^# | grep "model=" | cut -d '=' -f 2 | tr '.' '-'`
mac=`cat /sys/class/net/wlan0/address | tr -d ':' | tr 'a-z' 'A-Z' | cut -c 9-12`
SSID=${model}"_miap"${mac}
PASSWORD="" #TODO: password arg
VENDOR_PARAMS="$3"
DRIVER="nl80211"
WPA_CONF="wpa_ap.conf"
HOSTAP_CONF="hostap.conf"
DHCP_CONF="udhcpd.conf"
STATIC_IP="192.168.5.1"
STATIC_MASK="255.255.255.0"
STATIC_GW="192.168.5.1"
DHCP_START_IP="192.168.5.100"
DHCP_END_IP="192.168.5.200"

rm stationmode &2>/dev/null

#############################
######### Functions #########
#############################

create_config_file_wifi()
{
    FILE="$1"
    WIFI_SSID="$2"
    WIFI_PASS="$3"
    WIFI_VENDOR_PARAMS="$4"

    #Delete file
    if [ -e "$FILE" ] ; then
        rm "$FILE"
    fi

    echo "# This file has been generated by HomeKit engine. DO NOT EDIT!
    ctrl_interface=/var/run/wpa_supplicant
    eapol_version=1
    ap_scan=1
    fast_reauth=1
    update_config=1
    " >> "$FILE"

    #Append vendor params if in args
    if [ $WIFI_VENDOR_PARAMS ] ; then
        echo "ap_vendor_elements=$WIFI_VENDOR_PARAMS
        " >> "$FILE"
    fi

    echo "network={
    ssid=\"$WIFI_SSID\"
    " >> "$FILE"

    #Append password if in args
    if [ $WIFI_PASS ] ; then
        echo "psk=\"$WIFI_PASS\"
        " >> "$FILE"
    fi

    echo "key_mgmt=NONE
    priority=10
    mode=2
    frequency=2422
    }" >> "$FILE"
}

create_config_file_hostap()
{
    FILE="$1"
    WIFI_SSID="$2"
    WIFI_PASS="$3"
    WIFI_VENDOR_PARAMS="$4"

    #Delete file
    if [ -e "$FILE" ] ; then
        rm "$FILE"
    fi

    echo "# This file has been generated by HomeKit engine. DO NOT EDIT!
interface=$INTERFACE
ctrl_interface=/var/run/hostapd
ssid=$WIFI_SSID
channel=6
" >> "$FILE"

    #Append vendor params if in args
    if [ $WIFI_VENDOR_PARAMS ] ; then
        echo "vendor_elements=$WIFI_VENDOR_PARAMS
" >> "$FILE"
    fi

    #Append password if in args
    if [ $WIFI_PASS ] ; then
        echo "wpa_passphrase=$WIFI_PASS
" >> "$FILE"
    fi

    echo "driver=nl80211
beacon_int=100
hw_mode=g
ieee80211n=1
wme_enabled=1
ht_capab=[SHORT-GI-20][SHORT-GI-40][HT40+]
wpa_key_mgmt=WPA-PSK
wpa_pairwise=CCMP
max_num_sta=8
wpa_group_rekey=86400
" >> "$FILE"
}

create_config_file_dhcp()
{
    FILE="$1"
    IF="$2"
    START_IP="$3"
    END_IP="$4"
    ROUTER_IP="$5"
    MASK=$STATIC_MASK

    #Delete file
    if [ -e "$FILE" ] ; then
        rm "$FILE"
    fi

    echo "# This file has been generated by HomeKit engine. DO NOT EDIT!
    start $START_IP
    end $END_IP
    interface $IF
    #max_leases 254
    #remaining  yes
    #auto_time  7200
    #decline_time       3600
    #conflict_time      3600
    #offer_time 60
    #min_lease  60
    option subnet $MASK 
    opt router $ROUTER_IP
    " > "$FILE"
}

print_help()
{
echo "
This file configures and starts Wireless AP mode.
Only used by HomeKit engine.

Command line arguments:
  $1 interface name
  $2 wireless ssid
  $3 vendor element
"
}

#############################
#########  Script  ##########
#############################

#Print help
if [ $1 == "--help" ]; then
    print_help
    exit 1
fi

#Check input params
if [ $# -ne 3 ]; then
    echo "Not enough params!" >&2
    print_help
    exit 1
fi

echo "Started script wifi_ap.sh" >&2

#Broadcast the audio
if [ $VENDOR_PARAMS"empty" != "empty" ]; then
    echo \{\"command\":\"play_repeat\",\"name\":\"/home/root/music/ap_mode.mp3\",\"repeat_count\":1,\"volume\":40\}>/tmp/fifo_music_event_cmd
fi

#Clean and restart interface and other daemons
./wifi_disconnect.sh $INTERFACE

echo "Continue script wifi_ap.sh" >&2

#Create configuration file for wpa_supplicant
create_config_file_wifi $WPA_CONF "$SSID" "$PASSWORD" $VENDOR_PARAMS
create_config_file_hostap $HOSTAP_CONF "$SSID" "$PASSWORD" $VENDOR_PARAMS

#Create configuration file for DHCP server
create_config_file_dhcp $DHCP_CONF $INTERFACE $DHCP_START_IP $DHCP_END_IP $STATIC_GW

#Set static IP of the interface and create multicast default route
ifconfig $INTERFACE $STATIC_IP netmask $STATIC_MASK
route add -net 224.0.0.0 netmask 224.0.0.0 "$INTERFACE"

#Start hostapd in AP mode
mkdir -p /var/run/hostapd
if [ -x /usr/sbin/hostapd ] ; then
    hostapd "$HOSTAP_CONF" -ddB 2> /dev/null
    OUT=$?
    echo "Status of script is $OUT" >&2
    if [ $OUT -ne 0 ] ; then
        echo "Tool hostapd exited with error." >&2
        killall wpa_supplicant 2> /dev/null
        exit 1
    fi
else
    echo "Tool hostapd not found." >&2
    exit 1
fi

#set route for multicast
route add -net 224.0.0.0 netmask 224.0.0.0 "$INTERFACE"
route add default dev wlan0
route add -host 255.255.255.255 dev wlan0

#Start DHCP server
if [ -x /usr/sbin/udhcpd -o -x /usr/sbin/udhcpd ] ; then
    udhcpd "$DHCP_CONF" 2> /dev/null
    OUT=$?
    if [ $OUT -ne 0 ] ; then
        echo "Tool udhcpd exited with error." >&2
        killall wpa_supplicant 2> /dev/null
        killall udhcpd 2> /dev/null
        exit 1
    fi
else
    echo "Tool udhcpd not found." >&2
    exit 1
fi

#wait 5 seconds
sleep 5

exit 0
