#!/bin/sh

# This file configuring and start Wireless client mode and connect to AP
# Command line arguments:
#   $1 interface name
#   $2 wireless ssid
#   $3 password

INTERFACE="$1"
SSID="$2"
PASSWORD="$3"
WPA_CONF="/home/root/hkbridge/wpa_client.conf"


create_config_file_wifi()
{
    FILE=$1
    WIFI_SSID=$2
    WIFI_PASS=$3

    #Delete file
    if [ -e "$FILE" ] ; then
        rm "$FILE"
    fi

    echo "# This file has been generated by HomeKit engine. DO NOT EDIT!
    ctrl_interface=/var/run/wpa_supplicant
    update_config=1
    ap_scan=1
    eapol_version=1
    network={
    scan_ssid=1
    mode=0
    priority=20
    ssid=\"$WIFI_SSID\"
    " >> "$FILE"
    
    #Append password if in args
    if [ "$WIFI_PASS" != "" ] ; then
        echo "    psk=\"$WIFI_PASS\"" >> "$FILE"
    else
        echo "    key_mgmt=NONE" >> "$FILE"
    fi
    
echo "}" >> "$FILE"
}

print_help()
{
    echo "
    This file create wpa_client.conf for homekit.

    Command line arguments:
      $1 interface name
      $2 wireless ssid
      $3 password
    "
}

#############################
#########  Script  ##########
#############################

#Print help
if [ $1 == "--help" ]; then
    print_help
    exit 1
fi

#Check input params
if [ $# -ne 3 ]; then
    echo "Not enough params!" >&2
    print_help    
    exit 1
fi

echo "Started script $0" >&2

#Broadcast the audio
if [ ! -f /home/root/hkbridge/stationmode ]; then
    touch /home/root/hkbridge/stationmode
    echo "touch /home/root/hkbridge/stationmode>>>>>>>>>"
    sync
fi

#Create configuration file for wpa_supplicant
create_config_file_wifi $WPA_CONF "$SSID" "$PASSWORD"

exit 0
