#!/bin/sh

WIFI_START_SCRIPT="/lumi/app/miio/wifi_start.sh"
MIIO_RECV_LINE="/lumi/app/miio/miio_recv_line"
MIIO_SEND_LINE="/lumi/app/miio/miio_send_line"
MIIO_CONF_DIR=/lumi/conf
YOUR_TIMEZONE_DIR=/usr/share/zoneinfo
YOUR_LINK_TIMEZONE_FILE=/etc/localtime

WIFI_MAX_RETRY=5
WIFI_RETRY_INTERVAL=3
WIFI_SSID=


# contains(string, substring)
#
# Returns 0 if the specified string contains the specified substring,
# otherwise returns 1.
contains() {
    string="$1"
    substring="$2"
    if test "${string#*$substring}" != "$string"
    then
        return 0    # $substring is in $string
    else
        return 1    # $substring is not in $string
    fi
}

send_helper_ready() {
	IPC_RETRY=1                                
	while [ $IPC_RETRY -lt 15 ]
	do           
	echo "---IPC_RETRY---"
	if [ ! -f "/tmp/ot_connected" ]; then
		let IPC_RETRY=$IPC_RETRY+1
		echo "IPC_RETRY1:   "$IPC_RETRY                      
		sleep 1                                              
	else                                                         
		echo "IPC_RETRY0:"$IPC_RETRY
		sleep 1
		break                           
	fi                                      
	done                          
    ready_msg="{\"method\":\"_internal.helper_ready\"}"
    echo $ready_msg
    $MIIO_SEND_LINE "$ready_msg"
}

req_wifi_conf_status() {
    wificonf_dir=$1
    wificonf_dir=${wificonf_dir##*params\":\"}
    wificonf_dir=${wificonf_dir%%\"*}
	wificonf_dir=${wificonf_dir//\\/}	  # remove '\'
    wificonf_file=${wificonf_dir}/wifi.conf

    REQ_WIFI_CONF_STATUS_RESPONSE=""
    if [ -e $wificonf_file ]; then
	REQ_WIFI_CONF_STATUS_RESPONSE="{\"method\":\"_internal.res_wifi_conf_status\",\"params\":1}"

	WIFI_SSID=`cat $wificonf_file | grep ssid`
	WIFI_SSID=${WIFI_SSID#*ssid=\"}
	WIFI_SSID=${WIFI_SSID%\"*}
	echo "WIFI_SSID: $WIFI_SSID"
    else
	REQ_WIFI_CONF_STATUS_RESPONSE="{\"method\":\"_internal.res_wifi_conf_status\",\"params\":0}"
    fi
}

request_dinfo() {
    dinfo_dir=$1
    dinfo_dir=${dinfo_dir##*params\":\"}
    dinfo_dir=${dinfo_dir%%\"*}
	dinfo_dir=${dinfo_dir//\\/}	  # remove '\'
    dinfo_file=${dinfo_dir}/device.conf

    dinfo_did=`cat $dinfo_file | grep -v ^# | grep ^did= | tail -1 | cut -d '=' -f 2`
    dinfo_key=`cat $dinfo_file | grep -v ^# | grep ^key= | tail -1 | cut -d '=' -f 2`
    dinfo_vendor=`cat $dinfo_file | grep -v ^# | grep ^vendor= | tail -1 | cut -d '=' -f 2`
    dinfo_mac=`cat $dinfo_file | grep -v ^# | grep ^mac= | tail -1 | cut -d '=' -f 2`
    dinfo_model=`cat $dinfo_file | grep -v ^# | grep ^model= | tail -1 | cut -d '=' -f 2`
    RESPONSE_DINFO="{\"method\":\"_internal.response_dinfo\",\"params\":{"
    if [ x$dinfo_did != x ]; then
	RESPONSE_DINFO="$RESPONSE_DINFO\"did\":$dinfo_did"
    fi
    if [ x$dinfo_key != x ]; then
	RESPONSE_DINFO="$RESPONSE_DINFO,\"key\":\"$dinfo_key\""
    fi
    if [ x$dinfo_vendor != x ]; then
	RESPONSE_DINFO="$RESPONSE_DINFO,\"vendor\":\"$dinfo_vendor\""
    fi
    if [ x$dinfo_mac != x ]; then
	RESPONSE_DINFO="$RESPONSE_DINFO,\"mac\":\"$dinfo_mac\""
    fi
    if [ x$dinfo_model != x ]; then
	RESPONSE_DINFO="$RESPONSE_DINFO,\"model\":\"$dinfo_model\""
    fi
    RESPONSE_DINFO="$RESPONSE_DINFO}}"
}

request_dtoken() {
    dtoken_string=$1
    dtoken_dir=${dtoken_string##*dir\":\"}
    dtoken_dir=${dtoken_dir%%\"*}
	dtoken_dir=${dtoken_dir//\\/}	  # remove '\'
    dtoken_token=${dtoken_string##*ntoken\":\"}
    dtoken_token=${dtoken_token%%\"*}

    dtoken_file=${dtoken_dir}/device.token
    dcountry_file=${dtoken_dir}/device.country

    if [ ! -e ${dtoken_dir}/wifi.conf ]; then
	rm -f ${dtoken_file}
    fi

    if [ -e ${dtoken_file} ]; then
	dtoken_token=`cat ${dtoken_file}`
    else
	echo ${dtoken_token} > ${dtoken_file}
    fi
    
    if [ -e ${dcountry_file} ]; then
	dcountry_country=`cat ${dcountry_file}`
    else
	dcountry_country=""
    fi

    RESPONSE_DTOKEN="{\"method\":\"_internal.response_dtoken\",\"params\":\"${dtoken_token}\"}"
    RESPONSE_DCOUNTRY="{\"method\":\"_internal.response_dcountry\",\"params\":\"${dcountry_country}\"}"
}

save_wifi_conf() {
    datadir=$1
    miio_ssid=$2
    miio_passwd=$3
    miio_uid=$4
    miio_country=$5
    if [ x"$miio_passwd" = x ]; then
	miio_key_mgmt="NONE"
    else
	miio_key_mgmt="WPA"
    fi

    echo ssid=\"$miio_ssid\" > $datadir/wifi.conf
    echo psk=\"$miio_passwd\" >> $datadir/wifi.conf
    echo key_mgmt=$miio_key_mgmt >> $datadir/wifi.conf
    echo uid=$miio_uid >> $datadir/wifi.conf
    echo $miio_uid > $datadir/device.uid
    echo $miio_country > $datadir/device.country

    # create wifi config file for homekit
    /lumi/app/miio/create_hk_wifi_conf.sh wlan0 "$miio_ssid" "$miio_passwd"
    pkill -10 hkbridge	# notify wifi conf change to hkbridge
}

clear_wifi_conf() {
    datadir=$1
    rm -f $datadir/wifi.conf
    rm -f $datadir/device.uid
    rm -f $datadir/device.country
}

save_tz_conf() {
	new_tz=$YOUR_TIMEZONE_DIR/$1
#	if [ -e $new_tz ]; then
	if [ -f $new_tz ]; then	 # exist & file
		ln -sf  $new_tz $YOUR_LINK_TIMEZONE_FILE
		echo "timezone set success:$new_tz"
	else
		echo "timezone is not exist:$new_tz"
	fi
}

sanity_check() {
    if [ ! -e $WIFI_START_SCRIPT ]; then
	echo "Can't find wifi_start.sh: $WIFI_START_SCRIPT"
	echo 'Please change $WIFI_START_SCRIPT'
	exit 1
    fi
}

main() {
    while true; do
	BUF=`$MIIO_RECV_LINE`
	if [ $? -ne 0 ]; then
	    sleep 1;
	    continue
	fi
	echo "miio_recv_line: $BUF"
#	BUF=`echo "$BUF" | jq -Mc .`            # 暂时只处理 _internal.wifi_start                             
#	if [ $? -ne 0 -o "$BUF" = "" ]; then
#		echo "json parse error! code:$?"                             
#		continue                                           
#	fi                                           
#	echo "after jq format: $BUF" 
	if contains "$BUF" "_internal.info"; then
	    STRING=`wpa_cli status`

	    ifname=${STRING#*\'}
	    ifname=${ifname%%\'*}
	    echo "ifname: $ifname"

	    if [ "x$WIFI_SSID" != "x" ]; then
		ssid=$WIFI_SSID
	    else
		ssid=${STRING##*ssid=}
		ssid=`echo ${ssid} | cut -d ' ' -f 1`
	    fi
	    echo "ssid: $ssid"

	    bssid=${STRING##*bssid=}
	    bssid=`echo ${bssid} | cut -d ' ' -f 1 | tr '[:lower:]' '[:upper:]'`
	    echo "bssid: $bssid"

	    ip=${STRING##*ip_address=}
		echo "==================================="                                              
        echo $STRING                                                                            
        echo "===================================" 
	    ip=`echo ${ip} | cut -d ' ' -f 1`
	    echo "ip: $ip"


		#rssi=`iwconfig ${ifname} | grep 'Signal level' | cut -f 2 -d ':' | cut -f 1 -d ' '`
		#rssi=`iwconfig wlan0 | grep 'Signal level' | cut -d = -f 3 | cut -f 1 -d ' '`;
		#rssi="100/100"
		rssi=`wpa_cli signal_poll | grep RSSI | cut -f 2 -d '='`
		if [ "x$rssi" = "x" ];then
			rssi=0
		fi
		echo "rssi: $rssi"

	    STRING=`ifconfig ${ifname}`

	    netmask=${STRING##*Mask:}
	    netmask=`echo ${netmask} | cut -d ' ' -f 1`
	    echo "netmask: $netmask"

	    gw=`route -n|grep 'UG'|tr -s ' ' | cut -f 2 -d ' '`
	    echo "gw: $gw"

	    # get vendor and then version
	    vendor=`grep "vendor" $MIIO_CONF_DIR/device.conf | cut -f 2 -d '=' | tr '[:lower:]' '[:upper:]'`
	    sw_version=`grep "${vendor}_VERSION" /etc/os-release | cut -f 2 -d '='`
	    if [ -z $sw_version ]; then
		sw_version="unknown"
	    fi
		
		mcu_fw_version=`grep MCU_FW_VERSION /etc/os-release | cut -f 2 -d '='`

		if [ -z $mcu_fw_version ]; then
		mcu_fw_version="unknown"
	    fi

	    msg="{\"method\":\"_internal.info\",\"partner_id\":\"\",\"params\":{\
\"hw_ver\":\"Linux\",\"fw_ver\":\"$sw_version\",\"mcu_fw_ver\":\"$mcu_fw_version\",\
\"ap\":{\
 \"ssid\":\"$ssid\",\"rssi\":\"$rssi\",\"bssid\":\"$bssid\"\
},\
\"netif\":{\
 \"localIp\":\"$ip\",\"mask\":\"$netmask\",\"gw\":\"$gw\"\
}}}"

	    echo $msg
	    $MIIO_SEND_LINE "$msg"
	elif contains "$BUF" "miIO.config_router"; then
		echo "=======max add this:Got _internal.req_wifi_conf_status"
		echo "$BUF"
		echo "======================================================"
	elif contains "$BUF" "_internal.req_wifi_conf_status"; then
	    echo "Got _internal.req_wifi_conf_status"
	    req_wifi_conf_status "$BUF"
	    echo $REQ_WIFI_CONF_STATUS_RESPONSE
	    $MIIO_SEND_LINE "$REQ_WIFI_CONF_STATUS_RESPONSE"
	elif contains "$BUF" "_internal.wifi_start"; then
		BUF=`echo "$BUF" | jq -Mc .`                                         
		if [ $? -ne 0 -o "$BUF" = "" ]; then
			echo "json parse error! code:$?"                             
			continue                                           
		fi                                           
        echo "after jq format: $BUF" 
	    wificonf_dir2=${BUF##*\"datadir\":\"}
	    wificonf_dir2=${wificonf_dir2%%\"*}
		wificonf_dir2=${wificonf_dir2//\\/}	  # remove '\'
		echo "wifi config file : ${wificonf_dir2}/wifi.conf" 
		
	    miio_ssid=${BUF##*\"ssid\":\"}
	    miio_ssid=${miio_ssid%%\",\"passwd\":\"*}		
		echo "miio_ssid : ${miio_ssid}" 
		WIFI_SSID=$miio_ssid
		
	    miio_passwd=${BUF##*\",\"passwd\":\"}
	    miio_passwd=${miio_passwd%%\",\"uid\":\"*}
		echo "miio_passwd : ${miio_passwd}" 
		
	    miio_uid=${BUF##*\",\"uid\":\"}
	    miio_uid=${miio_uid%%\"*}
	    miio_country=${BUF##*\",\"country_domain\":\"}
	    #miio_country=${miio_country%%\"*}
	    miio_country=${miio_country%%\",\"tz\":\"*}
	    tz=${BUF##*\",\"tz\":\"}
	    tz=${tz%%\"*}
		tz=${tz//\\/}	  # remove '\'
		
		#gmt_offset=${BUF##*\",\"gmt_offset\":}
	    #gmt_offset=${gmt_offset%%,*}
		
		#echo "hello max buf: $BUF" 
		#echo "hello max tz: $tz" 
		#echo "hello max gmt_offset: $gmt_offset" 
		
	    save_wifi_conf "$wificonf_dir2" "$miio_ssid" "$miio_passwd" "$miio_uid" "$miio_country"
	    save_tz_conf "$tz"

	    CMD=$WIFI_START_SCRIPT
	    RETRY=1
	    WIFI_SUCC=1
		WIFI_ESSID="`iwconfig wlan0 | grep ESSID`"
		WIFI_ESSID=${WIFI_ESSID##*ESSID\:\"}
		WIFI_ESSID=${WIFI_ESSID%%\"*}
		echo "get ESSID:$WIFI_ESSID"
		if [ "$WIFI_ESSID" = "" -o "$WIFI_ESSID" != "$miio_ssid" ]; then
			until [ $RETRY -gt $WIFI_MAX_RETRY ]
			do
				WIFI_SUCC=1
				echo "Retry $RETRY: CMD=${CMD}"
				${CMD} && break
				WIFI_SUCC=0

				if [ $WIFI_MAX_RETRY -eq 1 ]; then
				   break
				fi
				let RETRY=$RETRY+1
				sleep $WIFI_RETRY_INTERVAL
			done
		fi

	    if [ $WIFI_SUCC -eq 1 ]; then
			msg="{\"method\":\"_internal.wifi_connected\"}"
			echo $msg
			$MIIO_SEND_LINE "$msg"
	    else
			clear_wifi_conf $wificonf_dir2
			CMD=$WIFI_START_SCRIPT
			echo "Back to AP mode, CMD=${CMD}"
			${CMD}
			msg="{\"method\":\"_internal.wifi_ap_mode\",\"params\":null}";
			echo $msg
			$MIIO_SEND_LINE "$msg"
	    fi
	elif contains "$BUF" "_internal.request_dinfo"; then
	    echo "Got _internal.request_dinfo"
	    request_dinfo "$BUF"
	    echo $RESPONSE_DINFO
	    $MIIO_SEND_LINE "$RESPONSE_DINFO"
	elif contains "$BUF" "_internal.request_dtoken"; then
	    echo "Got _internal.request_dtoken"
	    request_dtoken "$BUF"
	    echo $RESPONSE_DTOKEN
	    $MIIO_SEND_LINE "$RESPONSE_DTOKEN"
		echo $RESPONSE_DCOUNTRY
	    $MIIO_SEND_LINE "$RESPONSE_DCOUNTRY"
	#elif contains "$BUF" "_internal.config_tz"; then
	#    echo "Got _internal.config_tz"
	    tz=${BUF##*\",\"tz\":\"}
	    tz=${tz%%\"*}
		tz=${tz//\\/}

	    save_tz_conf "$tz"
	else
	    echo "Unknown cmd: $BUF"
	fi
    done
}

sanity_check
send_helper_ready
main
